@@ -11,15 +11,10 @@
 
 /*** declarations of public functions ************************************************************/
 
-#if ! GLIB_CHECK_VERSION (2, 28, 0)
-void g_slist_free_full (GSList * list, GDestroyNotify free_func);
-void g_list_free_full (GList * list, GDestroyNotify free_func);
-#endif /* ! GLIB_CHECK_VERSION (2, 28, 0) */
-
-#if ! GLIB_CHECK_VERSION (2, 64, 0)
+#if ! GLIB_CHECK_VERSION (2, 63, 3)
 void g_clear_slist (GSList ** slist_ptr, GDestroyNotify destroy);
 void g_clear_list (GList ** list_ptr, GDestroyNotify destroy);
-#endif /* ! GLIB_CHECK_VERSION (2, 64, 0) */
+#endif /* ! GLIB_CHECK_VERSION (2, 63, 3) */
 
 #if ! GLIB_CHECK_VERSION (2, 32, 0)
 void g_queue_free_full (GQueue * queue, GDestroyNotify free_func);
