@@ -31,12 +31,13 @@ typedef mc_search_cbret_t (*mc_update_fn) (const void *user_data, gsize char_off
 
 typedef enum
 {
-    MC_SEARCH_E_OK,
+    MC_SEARCH_E_OK = 0,
     MC_SEARCH_E_INPUT,
     MC_SEARCH_E_REGEX_COMPILE,
     MC_SEARCH_E_REGEX,
     MC_SEARCH_E_REGEX_REPLACE,
-    MC_SEARCH_E_NOTFOUND
+    MC_SEARCH_E_NOTFOUND,
+    MC_SEARCH_E_ABORT
 } mc_search_error_t;
 
 typedef enum
@@ -53,7 +54,8 @@ enum mc_search_cbret_t
     MC_SEARCH_CB_OK = 0,
     MC_SEARCH_CB_INVALID = -1,
     MC_SEARCH_CB_ABORT = -2,
-    MC_SEARCH_CB_SKIP = -3
+    MC_SEARCH_CB_SKIP = -3,
+    MC_SEARCH_CB_NOTFOUND = -4
 };
 
 /*** structures declarations (and typedefs of structures)*****************************************/
@@ -165,4 +167,9 @@ gboolean mc_search (const gchar * pattern, const gchar * pattern_charset, const
 int mc_search_getstart_result_by_num (mc_search_t *, int);
 int mc_search_getend_result_by_num (mc_search_t *, int);
 
+/* *INDENT-OFF* */
+void mc_search_set_error (mc_search_t * lc_mc_search, mc_search_error_t code, const gchar * format, ...)
+     G_GNUC_PRINTF (3, 4);
+/* *INDENT-ON* */
+
 #endif
