@@ -1,7 +1,7 @@
 /*
    Directory cache support
 
-   Copyright (C) 1998-2019
+   Copyright (C) 1998-2020
    Free Software Foundation, Inc.
 
    Written by:
@@ -63,7 +63,6 @@
 #ifdef HAVE_SYS_SELECT_H
 #include <sys/select.h>
 #endif
-#include <sys/time.h>           /* gettimeofday() */
 #include <sys/types.h>
 #include <unistd.h>
 
@@ -71,6 +70,7 @@
 
 #include "lib/tty/tty.h"        /* enable/disable interrupt key */
 #include "lib/util.h"           /* custom_canonicalize_pathname() */
+#include "lib/timer.h"
 #if 0
 #include "lib/widget.h"         /* message() */
 #endif
@@ -860,10 +860,10 @@ vfs_s_free (vfsid id)
 
 /* --------------------------------------------------------------------------------------------- */
 
-static int
+static gboolean
 vfs_s_dir_uptodate (struct vfs_class *me, struct vfs_s_inode *ino)
 {
-    struct timeval tim;
+    guint64 tim;
 
     if (me->flush)
     {
@@ -871,9 +871,9 @@ vfs_s_dir_uptodate (struct vfs_class *me, struct vfs_s_inode *ino)
         return 0;
     }
 
-    gettimeofday (&tim, NULL);
+    tim = mc_timer_elapsed (mc_global.timer);
 
-    return (tim.tv_sec < ino->timestamp.tv_sec ? 1 : 0);
+    return (tim < ino->timestamp);
 }
 
 /* --------------------------------------------------------------------------------------------- */
