@@ -1,7 +1,7 @@
 /*
    Widgets for the Midnight Commander
 
-   Copyright (C) 1994-2019
+   Copyright (C) 1994-2020
    Free Software Foundation, Inc.
 
    Authors:
@@ -46,13 +46,10 @@
 #include "lib/skin.h"
 #include "lib/strutil.h"
 #include "lib/util.h"
-#include "lib/keybind.h"        /* global_keymap_t */
 #include "lib/widget.h"
 #include "lib/event.h"          /* mc_event_raise() */
 #include "lib/mcconfig.h"       /* mc_config_history_*() */
 
-#include "input_complete.h"
-
 /*** global variables ****************************************************************************/
 
 gboolean quote = FALSE;
@@ -180,7 +177,10 @@ do_show_hist (WInput * in)
                              g_list_position (in->history.list, in->history.list));
     history_show (&hd);
 
+    /* in->history.list was destroyed in history_show().
+     * Apply new history and current postition to avoid use-after-free. */
     in->history.list = hd.list;
+    in->history.current = in->history.list;
     if (hd.text != NULL)
     {
         input_assign_text (in, hd.text);
@@ -806,7 +806,7 @@ input_execute_cmd (WInput * in, long command)
         do_show_hist (in);
         break;
     case CK_Complete:
-        complete (in);
+        input_complete (in);
         break;
     default:
         res = MSG_NOT_HANDLED;
@@ -871,7 +871,7 @@ input_save_history (const gchar * event_group_name, const gchar * event_name,
     (void) event_group_name;
     (void) event_name;
 
-    if (!in->is_password && (WIDGET (in)->owner->ret_value != B_CANCEL))
+    if (!in->is_password && (DIALOG (WIDGET (in)->owner)->ret_value != B_CANCEL))
     {
         ev_history_load_save_t *ev = (ev_history_load_save_t *) data;
 
@@ -895,7 +895,7 @@ input_destroy (WInput * in)
         exit (EXIT_FAILURE);
     }
 
-    input_free_completions (in);
+    input_complete_free (in);
 
     /* clean history */
     if (in->history.list != NULL)
@@ -995,6 +995,7 @@ input_new (int y, int x, const int *colors, int width, const char *def_text,
     w = WIDGET (in);
     widget_init (w, y, x, 1, width, input_callback, input_mouse_callback);
     w->options |= WOP_SELECTABLE | WOP_IS_INPUT | WOP_WANT_CURSOR;
+    w->keymap = input_map;
 
     in->color = colors;
     in->first = TRUE;
@@ -1039,15 +1040,16 @@ cb_ret_t
 input_callback (Widget * w, Widget * sender, widget_msg_t msg, int parm, void *data)
 {
     WInput *in = INPUT (w);
+    WDialog *h = DIALOG (w->owner);
     cb_ret_t v;
 
     switch (msg)
     {
     case MSG_INIT:
         /* subscribe to "history_load" event */
-        mc_event_add (w->owner->event_group, MCEVENT_HISTORY_LOAD, input_load_history, w, NULL);
+        mc_event_add (h->event_group, MCEVENT_HISTORY_LOAD, input_load_history, w, NULL);
         /* subscribe to "history_save" event */
-        mc_event_add (w->owner->event_group, MCEVENT_HISTORY_SAVE, input_save_history, w, NULL);
+        mc_event_add (h->event_group, MCEVENT_HISTORY_SAVE, input_save_history, w, NULL);
         if (in->label != NULL)
             widget_set_state (WIDGET (in->label), WST_DISABLED, widget_get_state (w, WST_DISABLED));
         return MSG_HANDLED;
@@ -1096,9 +1098,9 @@ input_callback (Widget * w, Widget * sender, widget_msg_t msg, int parm, void *d
 
     case MSG_DESTROY:
         /* unsubscribe from "history_load" event */
-        mc_event_del (w->owner->event_group, MCEVENT_HISTORY_LOAD, input_load_history, w);
+        mc_event_del (h->event_group, MCEVENT_HISTORY_LOAD, input_load_history, w);
         /* unsubscribe from "history_save" event */
-        mc_event_del (w->owner->event_group, MCEVENT_HISTORY_SAVE, input_save_history, w);
+        mc_event_del (h->event_group, MCEVENT_HISTORY_SAVE, input_save_history, w);
         input_destroy (in);
         return MSG_HANDLED;
 
@@ -1128,28 +1130,27 @@ input_handle_char (WInput * in, int key)
 
     if (quote)
     {
-        input_free_completions (in);
+        input_complete_free (in);
         v = insert_char (in, key);
         input_update (in, TRUE);
         quote = FALSE;
         return v;
     }
 
-    command = keybind_lookup_keymap_command (input_map, key);
-
+    command = widget_lookup_key (WIDGET (in), key);
     if (command == CK_IgnoreKey)
     {
         if (key > 255)
             return MSG_NOT_HANDLED;
         if (in->first)
             port_region_marked_for_delete (in);
-        input_free_completions (in);
+        input_complete_free (in);
         v = insert_char (in, key);
     }
     else
     {
         if (command != CK_Complete)
-            input_free_completions (in);
+            input_complete_free (in);
         input_execute_cmd (in, command);
         v = MSG_HANDLED;
         if (in->first)
@@ -1162,25 +1163,6 @@ input_handle_char (WInput * in, int key)
 
 /* --------------------------------------------------------------------------------------------- */
 
-/* This function is a test for a special input key used in complete.c */
-/* Returns 0 if it is not a special key, 1 if it is a non-complete key
-   and 2 if it is a complete key */
-int
-input_key_is_in_map (WInput * in, int key)
-{
-    long command;
-
-    (void) in;
-
-    command = keybind_lookup_keymap_command (input_map, key);
-    if (command == CK_IgnoreKey)
-        return 0;
-
-    return (command == CK_Complete) ? 2 : 1;
-}
-
-/* --------------------------------------------------------------------------------------------- */
-
 void
 input_assign_text (WInput * in, const char *text)
 {
@@ -1190,7 +1172,7 @@ input_assign_text (WInput * in, const char *text)
     if (text == NULL)
         text = "";
 
-    input_free_completions (in);
+    input_complete_free (in);
     in->mark = -1;
     in->need_push = TRUE;
     in->charpoint = 0;
@@ -1238,7 +1220,7 @@ input_set_point (WInput * in, int pos)
     max_pos = str_length (in->buffer);
     pos = MIN (pos, max_pos);
     if (pos != in->point)
-        input_free_completions (in);
+        input_complete_free (in);
     in->point = pos;
     in->charpoint = 0;
     input_update (in, TRUE);
@@ -1381,17 +1363,8 @@ input_clean (WInput * in)
     in->point = 0;
     in->charpoint = 0;
     in->mark = -1;
-    input_free_completions (in);
+    input_complete_free (in);
     input_update (in, FALSE);
 }
 
 /* --------------------------------------------------------------------------------------------- */
-
-void
-input_free_completions (WInput * in)
-{
-    g_strfreev (in->completions);
-    in->completions = NULL;
-}
-
-/* --------------------------------------------------------------------------------------------- */
