@@ -1,7 +1,7 @@
 /*
    Widget based utility functions.
 
-   Copyright (C) 1994-2019
+   Copyright (C) 1994-2020
    Free Software Foundation, Inc.
 
    Authors:
@@ -74,6 +74,7 @@ query_default_callback (Widget * w, Widget * sender, widget_msg_t msg, int parm,
         {
             WDialog *prev_dlg = NULL;
             int ypos, xpos;
+            WRect r;
 
             /* get dialog under h */
             if (top_dlg != NULL)
@@ -101,9 +102,9 @@ query_default_callback (Widget * w, Widget * sender, widget_msg_t msg, int parm,
             xpos = COLS / 2 - w->cols / 2;
 
             /* set position */
-            dlg_set_position (h, ypos, xpos, w->lines, w->cols);
+            rect_init (&r, ypos, xpos, w->lines, w->cols);
 
-            return MSG_HANDLED;
+            return dlg_default_callback (w, NULL, MSG_RESIZE, 0, &r);
         }
         MC_FALLTHROUGH;
 
@@ -273,6 +274,7 @@ query_dialog (const char *header, const char *text, int flags, int count, ...)
 {
     va_list ap;
     WDialog *query_dlg;
+    WGroup *g;
     WButton *button;
     int win_len = 0;
     int i;
@@ -291,6 +293,7 @@ query_dialog (const char *header, const char *text, int flags, int count, ...)
         for (i = 0; i < count; i++)
         {
             char *cp = va_arg (ap, char *);
+
             win_len += str_term_width1 (cp) + 6;
             if (strchr (cp, '&') != NULL)
                 win_len--;
@@ -307,14 +310,15 @@ query_dialog (const char *header, const char *text, int flags, int count, ...)
     query_dlg =
         dlg_create (TRUE, 0, 0, lines, cols, pos_flags, FALSE, query_colors, query_default_callback,
                     NULL, "[QueryBox]", header);
+    g = GROUP (query_dlg);
 
     if (count > 0)
     {
         WButton *defbutton = NULL;
 
-        add_widget_autopos (query_dlg, label_new (2, 3, text), WPOS_KEEP_TOP | WPOS_CENTER_HORZ,
-                            NULL);
-        add_widget (query_dlg, hline_new (lines - 4, -1, -1));
+        group_add_widget_autopos (g, label_new (2, 3, text), WPOS_KEEP_TOP | WPOS_CENTER_HORZ,
+                                  NULL);
+        group_add_widget (g, hline_new (lines - 4, -1, -1));
 
         cols = (cols - win_len - 2) / 2 + 2;
         va_start (ap, count);
@@ -329,7 +333,7 @@ query_dialog (const char *header, const char *text, int flags, int count, ...)
                 xpos--;
 
             button = button_new (lines - 3, cols, B_USER + i, NORMAL_BUTTON, cur_name, NULL);
-            add_widget (query_dlg, button);
+            group_add_widget (g, button);
             cols += xpos;
             if (i == sel_pos)
                 defbutton = button;
@@ -356,9 +360,9 @@ query_dialog (const char *header, const char *text, int flags, int count, ...)
     }
     else
     {
-        add_widget_autopos (query_dlg, label_new (2, 3, text), WPOS_KEEP_TOP | WPOS_CENTER_HORZ,
-                            NULL);
-        add_widget (query_dlg, button_new (0, 0, 0, HIDDEN_BUTTON, "-", NULL));
+        group_add_widget_autopos (g, label_new (2, 3, text), WPOS_KEEP_TOP | WPOS_CENTER_HORZ,
+                                  NULL);
+        group_add_widget (g, button_new (0, 0, 0, HIDDEN_BUTTON, "-", NULL));
         last_query_dlg = query_dlg;
     }
     sel_pos = 0;
@@ -687,6 +691,7 @@ simple_status_msg_init_cb (status_msg_t * sm)
 {
     simple_status_msg_t *ssm = SIMPLE_STATUS_MSG (sm);
     Widget *wd = WIDGET (sm->dlg);
+    WGroup *wg = GROUP (sm->dlg);
 
     const char *b_name = N_("&Abort");
     int b_width;
@@ -702,10 +707,10 @@ simple_status_msg_init_cb (status_msg_t * sm)
 
     y = 2;
     ssm->label = label_new (y++, 3, "");
-    add_widget_autopos (sm->dlg, ssm->label, WPOS_KEEP_TOP | WPOS_CENTER_HORZ, NULL);
-    add_widget (sm->dlg, hline_new (y++, -1, -1));
+    group_add_widget_autopos (wg, ssm->label, WPOS_KEEP_TOP | WPOS_CENTER_HORZ, NULL);
+    group_add_widget (wg, hline_new (y++, -1, -1));
     b = WIDGET (button_new (y++, 3, B_CANCEL, NORMAL_BUTTON, b_name, NULL));
-    add_widget_autopos (sm->dlg, b, WPOS_KEEP_TOP | WPOS_CENTER_HORZ, NULL);
+    group_add_widget_autopos (wg, b, WPOS_KEEP_TOP | WPOS_CENTER_HORZ, NULL);
 
     widget_set_size (wd, wd->y, wd->x, y + 2, wd_width);
 }
