@@ -1,7 +1,7 @@
 /*
    Editor syntax highlighting.
 
-   Copyright (C) 1996-2019
+   Copyright (C) 1996-2020
    Free Software Foundation, Inc.
 
    Written by:
@@ -56,7 +56,7 @@
 #include "lib/global.h"
 #include "lib/search.h"         /* search engine */
 #include "lib/skin.h"
-#include "lib/fileloc.h"        /* EDIT_DIR, EDIT_SYNTAX_FILE */
+#include "lib/fileloc.h"        /* EDIT_HOME_DIR, EDIT_HOME_SYNTAX_FILE */
 #include "lib/strutil.h"        /* utf string functions */
 #include "lib/util.h"
 #include "lib/widget.h"         /* message() */
@@ -883,7 +883,7 @@ open_include_file (const char *filename)
 
     g_free (error_file_name);
     error_file_name =
-        g_build_filename (mc_config_get_data_path (), EDIT_DIR, filename, (char *) NULL);
+        g_build_filename (mc_config_get_data_path (), EDIT_HOME_DIR, filename, (char *) NULL);
     f = fopen (error_file_name, "r");
     if (f != NULL)
         return f;
@@ -1486,7 +1486,7 @@ edit_load_syntax (WEdit * edit, GPtrArray * pnames, const char *type)
     if (edit != NULL && edit->filename_vpath == NULL)
         return;
 
-    f = mc_config_get_full_path (EDIT_SYNTAX_FILE);
+    f = mc_config_get_full_path (EDIT_HOME_SYNTAX_FILE);
     if (edit != NULL)
         r = edit_read_syntax_file (edit, pnames, f, vfs_path_as_str (edit->filename_vpath),
                                    get_first_editor_line (edit),
