@@ -1,7 +1,7 @@
 /*
    Execution routines for GNU Midnight Commander
 
-   Copyright (C) 2003-2019
+   Copyright (C) 2003-2020
    Free Software Foundation, Inc.
 
    Written by:
@@ -461,7 +461,7 @@ toggle_subshell (void)
     {
         if (message_flag)
             message (D_ERROR, MSG_ERROR,
-                     _("Not an xterm or Linux console;\nthe panels cannot be toggled."));
+                     _("Not an xterm or Linux console;\nthe subshell cannot be toggled."));
         message_flag = FALSE;
         return;
     }
@@ -554,13 +554,6 @@ toggle_subshell (void)
             do_load_prompt ();
             if (new_dir_vpath != NULL)
                 do_possible_cd (new_dir_vpath);
-            if (mc_global.tty.console_flag != '\0' && output_lines != 0)
-            {
-                unsigned char end_line;
-
-                end_line = LINES - (mc_global.keybar_visible ? 1 : 0) - 1;
-                show_console_contents (output_start_y, end_line - output_lines, end_line);
-            }
         }
         else if (new_dir_vpath != NULL && mc_chdir (new_dir_vpath) != -1)
             vfs_setup_cwd ();
