@@ -1,7 +1,7 @@
 /*
    Some misc dialog boxes for the program.
 
-   Copyright (C) 1994-2019
+   Copyright (C) 1994-2020
    Free Software Foundation, Inc.
 
    Written by:
@@ -136,7 +136,7 @@ configure_callback (Widget * w, Widget * sender, widget_msg_t msg, int parm, voi
             Widget *ww;
 
             /* input line */
-            ww = dlg_find_by_id (DIALOG (w), configure_time_out_id);
+            ww = widget_find_by_id (w, configure_time_out_id);
             widget_disable (ww, not_single);
 
             return MSG_HANDLED;
@@ -192,12 +192,13 @@ skin_dlg_callback (Widget * w, Widget * sender, widget_msg_t msg, int parm, void
             WDialog *d = DIALOG (w);
             Widget *wd = WIDGET (d->data);
             int y, x;
+            WRect r;
 
             y = wd->y + (wd->lines - w->lines) / 2;
             x = wd->x + wd->cols / 2;
-            dlg_set_position (d, y, x, w->lines, w->cols);
+            rect_init (&r, y, x, w->lines, w->cols);
 
-            return MSG_HANDLED;
+            return dlg_default_callback (w, NULL, MSG_RESIZE, 0, &r);
         }
 
     default:
@@ -250,7 +251,7 @@ sel_skin_button (WButton * button, int action)
     }
 
     /* make list stick to all sides of dialog, effectively make it be resized with dialog */
-    add_widget_autopos (skin_dlg, skin_list, WPOS_KEEP_ALL, NULL);
+    group_add_widget_autopos (GROUP (skin_dlg), skin_list, WPOS_KEEP_ALL, NULL);
 
     result = dlg_run (skin_dlg);
     if (result == B_ENTER)
@@ -274,8 +275,6 @@ sel_skin_button (WButton * button, int action)
 static cb_ret_t
 panel_listing_callback (Widget * w, Widget * sender, widget_msg_t msg, int parm, void *data)
 {
-    WDialog *h = DIALOG (w);
-
     switch (msg)
     {
     case MSG_NOTIFY:
@@ -284,10 +283,10 @@ panel_listing_callback (Widget * w, Widget * sender, widget_msg_t msg, int parm,
             WCheck *ch;
             WInput *in1, *in2, *in3;
 
-            in1 = INPUT (dlg_find_by_id (h, panel_user_format_id));
-            in2 = INPUT (dlg_find_by_id (h, panel_brief_cols_id));
-            ch = CHECK (dlg_find_by_id (h, mini_user_status_id));
-            in3 = INPUT (dlg_find_by_id (h, mini_user_format_id));
+            in1 = INPUT (widget_find_by_id (w, panel_user_format_id));
+            in2 = INPUT (widget_find_by_id (w, panel_brief_cols_id));
+            ch = CHECK (widget_find_by_id (w, mini_user_status_id));
+            in3 = INPUT (widget_find_by_id (w, mini_user_format_id));
 
             if (!ch->state)
                 input_assign_text (in3, status_format[RADIO (sender)->sel]);
@@ -303,7 +302,7 @@ panel_listing_callback (Widget * w, Widget * sender, widget_msg_t msg, int parm,
         {
             WInput *in;
 
-            in = INPUT (dlg_find_by_id (h, mini_user_format_id));
+            in = INPUT (widget_find_by_id (w, mini_user_format_id));
 
             if (CHECK (sender)->state)
             {
@@ -314,7 +313,7 @@ panel_listing_callback (Widget * w, Widget * sender, widget_msg_t msg, int parm,
             {
                 WRadio *r;
 
-                r = RADIO (dlg_find_by_id (h, panel_list_formats_id));
+                r = RADIO (widget_find_by_id (w, panel_list_formats_id));
                 widget_disable (WIDGET (in), TRUE);
                 input_assign_text (in, status_format[r->sel]);
             }
@@ -355,7 +354,7 @@ sel_charset_button (WButton * button, int action)
             cpname = _("7-bit ASCII");  /* FIXME */
 
         button_set_text (button, cpname);
-        dlg_draw (WIDGET (button)->owner);
+        widget_draw (WIDGET (WIDGET (button)->owner));
     }
 
     return 0;
@@ -373,10 +372,12 @@ tree_callback (Widget * w, Widget * sender, widget_msg_t msg, int parm, void *da
     {
     case MSG_RESIZE:
         {
+            WRect r;
             Widget *bar;
 
-            /* simply call dlg_set_size() with new size */
-            dlg_set_size (h, LINES - 9, COLS - 20);
+            rect_init (&r, w->y, w->x, LINES - 9, COLS - 20);
+            dlg_default_callback (w, NULL, MSG_RESIZE, 0, &r);
+
             bar = WIDGET (find_buttonbar (h));
             bar->x = 0;
             bar->y = LINES - 1;
@@ -407,7 +408,7 @@ confvfs_callback (Widget * w, Widget * sender, widget_msg_t msg, int parm, void
             Widget *wi;
 
             /* input */
-            wi = dlg_find_by_id (DIALOG (w), ftpfs_proxy_host_id);
+            wi = widget_find_by_id (w, ftpfs_proxy_host_id);
             widget_disable (wi, not_use);
             return MSG_HANDLED;
         }
@@ -484,7 +485,7 @@ task_cb (WButton * button, int action)
     jobs_fill_listbox (bg_list);
 
     /* This can be optimized to just redraw this widget :-) */
-    dlg_draw (WIDGET (button)->owner);
+    widget_draw (WIDGET (WIDGET (button)->owner));
 
     return 0;
 }
@@ -1024,6 +1025,7 @@ tree_box (const char *current_dir)
 {
     WTree *mytree;
     WDialog *dlg;
+    WGroup *g;
     Widget *wd;
     char *val = NULL;
     WButtonBar *bar;
@@ -1033,13 +1035,14 @@ tree_box (const char *current_dir)
     /* Create the components */
     dlg = dlg_create (TRUE, 0, 0, LINES - 9, COLS - 20, WPOS_CENTER, FALSE, dialog_colors,
                       tree_callback, NULL, "[Directory Tree]", _("Directory tree"));
+    g = GROUP (dlg);
     wd = WIDGET (dlg);
 
     mytree = tree_new (2, 2, wd->lines - 6, wd->cols - 5, FALSE);
-    add_widget_autopos (dlg, mytree, WPOS_KEEP_ALL, NULL);
-    add_widget_autopos (dlg, hline_new (wd->lines - 4, 1, -1), WPOS_KEEP_BOTTOM, NULL);
+    group_add_widget_autopos (g, mytree, WPOS_KEEP_ALL, NULL);
+    group_add_widget_autopos (g, hline_new (wd->lines - 4, 1, -1), WPOS_KEEP_BOTTOM, NULL);
     bar = buttonbar_new (TRUE);
-    add_widget (dlg, bar);
+    group_add_widget (g, bar);
     /* restore ButtonBar coordinates after add_widget() */
     WIDGET (bar)->x = 0;
     WIDGET (bar)->y = LINES - 1;
@@ -1232,6 +1235,7 @@ jobs_box (void)
     const size_t n_but = G_N_ELEMENTS (job_but);
 
     WDialog *jobs_dlg;
+    WGroup *g;
     int cols = 60;
     int lines = 15;
     int x = 0;
@@ -1253,19 +1257,19 @@ jobs_box (void)
 
     jobs_dlg = dlg_create (TRUE, 0, 0, lines, cols, WPOS_CENTER, FALSE, dialog_colors, NULL, NULL,
                            "[Background jobs]", _("Background jobs"));
+    g = GROUP (jobs_dlg);
 
     bg_list = listbox_new (2, 2, lines - 6, cols - 6, FALSE, NULL);
     jobs_fill_listbox (bg_list);
-    add_widget (jobs_dlg, bg_list);
+    group_add_widget (g, bg_list);
 
-    add_widget (jobs_dlg, hline_new (lines - 4, -1, -1));
+    group_add_widget (g, hline_new (lines - 4, -1, -1));
 
     x = (cols - x) / 2;
     for (i = 0; i < n_but; i++)
     {
-        add_widget (jobs_dlg,
-                    button_new (lines - 3, x, job_but[i].value, job_but[i].flags, job_but[i].name,
-                                job_but[i].callback));
+        group_add_widget (g, button_new (lines - 3, x, job_but[i].value, job_but[i].flags,
+                                         job_but[i].name, job_but[i].callback));
         x += job_but[i].len + 1;
     }
 
