@@ -1,7 +1,7 @@
 /*
    Chown command -- for the Midnight Commander
 
-   Copyright (C) 1994-2019
+   Copyright (C) 1994-2020
    Free Software Foundation, Inc.
 
    This file is part of the Midnight Commander.
@@ -117,7 +117,7 @@ static WListbox *l_user, *l_group;
 /* --------------------------------------------------------------------------------------------- */
 
 static void
-chown_i18n (void)
+chown_init (void)
 {
     static gboolean i18n = FALSE;
     int i;
@@ -148,13 +148,11 @@ chown_i18n (void)
 /* --------------------------------------------------------------------------------------------- */
 
 static void
-chown_refresh (WDialog * h)
+chown_refresh (const Widget * h)
 {
     int y = 3;
     int x = 7 + GW * 2;
 
-    dlg_default_repaint (h);
-
     tty_setcolor (COLOR_NORMAL);
 
     widget_gotoyx (h, y + 0, x);
@@ -172,26 +170,28 @@ chown_refresh (WDialog * h)
 /* --------------------------------------------------------------------------------------------- */
 
 static cb_ret_t
-chown_callback (Widget * w, Widget * sender, widget_msg_t msg, int parm, void *data)
+chown_bg_callback (Widget * w, Widget * sender, widget_msg_t msg, int parm, void *data)
 {
     switch (msg)
     {
     case MSG_DRAW:
-        chown_refresh (DIALOG (w));
+        frame_callback (w, NULL, MSG_DRAW, 0, NULL);
+        chown_refresh (WIDGET (w->owner));
         return MSG_HANDLED;
 
     default:
-        return dlg_default_callback (w, sender, msg, parm, data);
+        return frame_callback (w, sender, msg, parm, data);
     }
 }
 
 /* --------------------------------------------------------------------------------------------- */
 
 static WDialog *
-chown_init (void)
+chown_dlg_create (void)
 {
     int single_set;
     WDialog *ch_dlg;
+    WGroup *g;
     int lines, cols;
     int i, y;
     struct passwd *l_pass;
@@ -202,12 +202,16 @@ chown_init (void)
     cols = GW * 3 + 2 + 6;
 
     ch_dlg =
-        dlg_create (TRUE, 0, 0, lines, cols, WPOS_CENTER, FALSE, dialog_colors, chown_callback,
-                    NULL, "[Chown]", _("Chown command"));
+        dlg_create (TRUE, 0, 0, lines, cols, WPOS_CENTER, FALSE, dialog_colors, NULL, NULL,
+                    "[Chown]", _("Chown command"));
+    g = GROUP (ch_dlg);
 
-    add_widget (ch_dlg, groupbox_new (2, 3, GH, GW, _("User name")));
+    /* draw background */
+    ch_dlg->bg->callback = chown_bg_callback;
+
+    group_add_widget (g, groupbox_new (2, 3, GH, GW, _("User name")));
     l_user = listbox_new (3, 4, GH - 2, GW - 2, FALSE, NULL);
-    add_widget (ch_dlg, l_user);
+    group_add_widget (g, l_user);
     /* add field for unknown names (numbers) */
     listbox_add_item (l_user, LISTBOX_APPEND_AT_END, 0, _("<Unknown user>"), NULL, FALSE);
     /* get and put user names in the listbox */
@@ -216,9 +220,9 @@ chown_init (void)
         listbox_add_item (l_user, LISTBOX_APPEND_SORTED, 0, l_pass->pw_name, NULL, FALSE);
     endpwent ();
 
-    add_widget (ch_dlg, groupbox_new (2, 4 + GW, GH, GW, _("Group name")));
+    group_add_widget (g, groupbox_new (2, 4 + GW, GH, GW, _("Group name")));
     l_group = listbox_new (3, 5 + GW, GH - 2, GW - 2, FALSE, NULL);
-    add_widget (ch_dlg, l_group);
+    group_add_widget (g, l_group);
     /* add field for unknown names (numbers) */
     listbox_add_item (l_group, LISTBOX_APPEND_AT_END, 0, _("<Unknown group>"), NULL, FALSE);
     /* get and put group names in the listbox */
@@ -227,42 +231,40 @@ chown_init (void)
         listbox_add_item (l_group, LISTBOX_APPEND_SORTED, 0, l_grp->gr_name, NULL, FALSE);
     endgrent ();
 
-    add_widget (ch_dlg, groupbox_new (2, 5 + GW * 2, GH, GW, _("File")));
+    group_add_widget (g, groupbox_new (2, 5 + GW * 2, GH, GW, _("File")));
     /* add widgets for the file information */
     for (i = 0; i < LABELS; i++)
     {
         chown_label[i].l = label_new (chown_label[i].y, 7 + GW * 2, "");
-        add_widget (ch_dlg, chown_label[i].l);
+        group_add_widget (g, chown_label[i].l);
     }
 
     if (single_set == 0)
     {
         int x;
 
-        add_widget (ch_dlg, hline_new (lines - chown_but[0].y - 1, -1, -1));
+        group_add_widget (g, hline_new (lines - chown_but[0].y - 1, -1, -1));
 
         y = lines - chown_but[0].y;
         x = (cols - blen) / 2;
 
         for (i = 0; i < BUTTONS - 2; i++)
         {
-            add_widget (ch_dlg,
-                        button_new (y, x, chown_but[i].ret_cmd, chown_but[i].flags,
-                                    chown_but[i].text, NULL));
+            group_add_widget (g, button_new (y, x, chown_but[i].ret_cmd, chown_but[i].flags,
+                                             chown_but[i].text, NULL));
             x += chown_but[i].len + 1;
         }
     }
 
     i = BUTTONS - 2;
     y = lines - chown_but[i].y;
-    add_widget (ch_dlg, hline_new (y - 1, -1, -1));
-    add_widget (ch_dlg,
-                button_new (y, WIDGET (ch_dlg)->cols / 2 - chown_but[i].len, chown_but[i].ret_cmd,
-                            chown_but[i].flags, chown_but[i].text, NULL));
+    group_add_widget (g, hline_new (y - 1, -1, -1));
+    group_add_widget (g, button_new (y, WIDGET (ch_dlg)->cols / 2 - chown_but[i].len,
+                                     chown_but[i].ret_cmd, chown_but[i].flags, chown_but[i].text,
+                                     NULL));
     i++;
-    add_widget (ch_dlg,
-                button_new (y, WIDGET (ch_dlg)->cols / 2 + 1, chown_but[i].ret_cmd,
-                            chown_but[i].flags, chown_but[i].text, NULL));
+    group_add_widget (g, button_new (y, WIDGET (ch_dlg)->cols / 2 + 1, chown_but[i].ret_cmd,
+                                     chown_but[i].flags, chown_but[i].text, NULL));
 
     /* select first listbox */
     widget_select (WIDGET (l_user));
@@ -395,7 +397,7 @@ chown_cmd (void)
     gboolean need_update;
     gboolean end_chown;
 
-    chown_i18n ();
+    chown_init ();
 
     current_file = 0;
     ignore_all = FALSE;
@@ -429,7 +431,7 @@ chown_cmd (void)
             break;
         }
 
-        ch_dlg = chown_init ();
+        ch_dlg = chown_dlg_create ();
 
         /* select in listboxes */
         listbox_select_entry (l_user, listbox_search_text (l_user, get_owner (sf_stat.st_uid)));
