@@ -1,7 +1,7 @@
 /*
    Panel managing.
 
-   Copyright (C) 1994-2019
+   Copyright (C) 1994-2020
    Free Software Foundation, Inc.
 
    Written by:
@@ -34,6 +34,9 @@
 #include <stdlib.h>
 #include <sys/stat.h>
 #include <inttypes.h>           /* PRIuMAX */
+#ifdef ENABLE_EXT2FS_ATTR
+#include <e2p/e2p.h>            /* fgetflags() */
+#endif
 
 #include "lib/global.h"
 #include "lib/unixcompat.h"
@@ -50,6 +53,10 @@
 #include "midnight.h"           /* the_menubar */
 #include "layout.h"
 #include "mountlist.h"
+#ifdef ENABLE_EXT2FS_ATTR
+#include "chattr.h"
+#endif
+
 #include "info.h"
 
 /*** global variables ****************************************************************************/
@@ -154,8 +161,8 @@ info_show_info (WInfo * info)
 
     default:
         MC_FALLTHROUGH;
-    case 16:
-        widget_gotoyx (w, 16, 3);
+    case 17:
+        widget_gotoyx (w, 17, 3);
         if ((myfs_stats.nfree == 0 && myfs_stats.nodes == 0) ||
             (myfs_stats.nfree == (uintmax_t) (-1) && myfs_stats.nodes == (uintmax_t) (-1)))
             tty_print_string (_("No node information"));
@@ -170,8 +177,8 @@ info_show_info (WInfo * info)
                         myfs_stats.nodes == 0 ? 0 :
                         (int) (100 * (long double) myfs_stats.nfree / myfs_stats.nodes));
         MC_FALLTHROUGH;
-    case 15:
-        widget_gotoyx (w, 15, 3);
+    case 16:
+        widget_gotoyx (w, 16, 3);
         if (myfs_stats.avail == 0 && myfs_stats.total == 0)
             tty_print_string (_("No space information"));
         else
@@ -185,41 +192,41 @@ info_show_info (WInfo * info)
                         (int) (100 * (long double) myfs_stats.avail / myfs_stats.total));
         }
         MC_FALLTHROUGH;
-    case 14:
-        widget_gotoyx (w, 14, 3);
+    case 15:
+        widget_gotoyx (w, 15, 3);
         tty_printf (_("Type:       %s"),
                     myfs_stats.typename ? myfs_stats.typename : _("non-local vfs"));
         if (myfs_stats.type != 0xffff && myfs_stats.type != -1)
             tty_printf (" (%Xh)", (unsigned int) myfs_stats.type);
         MC_FALLTHROUGH;
-    case 13:
-        widget_gotoyx (w, 13, 3);
+    case 14:
+        widget_gotoyx (w, 14, 3);
         str_printf (buff, _("Device:     %s"),
                     str_trunc (myfs_stats.device, w->cols - i18n_adjust));
         tty_print_string (buff->str);
         g_string_set_size (buff, 0);
         MC_FALLTHROUGH;
-    case 12:
-        widget_gotoyx (w, 12, 3);
+    case 13:
+        widget_gotoyx (w, 13, 3);
         str_printf (buff, _("Filesystem: %s"),
                     str_trunc (myfs_stats.mpoint, w->cols - i18n_adjust));
         tty_print_string (buff->str);
         g_string_set_size (buff, 0);
         MC_FALLTHROUGH;
+    case 12:
+        widget_gotoyx (w, 12, 3);
+        str_printf (buff, _("Accessed:   %s"), file_date (st.st_atime));
+        tty_print_string (buff->str);
+        g_string_set_size (buff, 0);
+        MC_FALLTHROUGH;
     case 11:
         widget_gotoyx (w, 11, 3);
-        str_printf (buff, _("Accessed:   %s"), file_date (st.st_atime));
+        str_printf (buff, _("Modified:   %s"), file_date (st.st_mtime));
         tty_print_string (buff->str);
         g_string_set_size (buff, 0);
         MC_FALLTHROUGH;
     case 10:
         widget_gotoyx (w, 10, 3);
-        str_printf (buff, _("Modified:   %s"), file_date (st.st_mtime));
-        tty_print_string (buff->str);
-        g_string_set_size (buff, 0);
-        MC_FALLTHROUGH;
-    case 9:
-        widget_gotoyx (w, 9, 3);
         /* The field st_ctime is changed by writing or by setting inode
            information (i.e., owner, group, link count, mode, etc.).  */
         /* TRANSLATORS: Time of last status change as in stat(2) man. */
@@ -227,8 +234,8 @@ info_show_info (WInfo * info)
         tty_print_string (buff->str);
         g_string_set_size (buff, 0);
         MC_FALLTHROUGH;
-    case 8:
-        widget_gotoyx (w, 8, 3);
+    case 9:
+        widget_gotoyx (w, 9, 3);
 #ifdef HAVE_STRUCT_STAT_ST_RDEV
         if (S_ISCHR (st.st_mode) || S_ISBLK (st.st_mode))
             tty_printf (_("Dev. type: major %lu, minor %lu"),
@@ -245,13 +252,36 @@ info_show_info (WInfo * info)
 #endif
         }
         MC_FALLTHROUGH;
+    case 8:
+        widget_gotoyx (w, 8, 3);
+        tty_printf (_("Owner:      %s/%s"), get_owner (st.st_uid), get_group (st.st_gid));
+        MC_FALLTHROUGH;
     case 7:
         widget_gotoyx (w, 7, 3);
-        tty_printf (_("Owner:      %s/%s"), get_owner (st.st_uid), get_group (st.st_gid));
+        tty_printf (_("Links:      %d"), (int) st.st_nlink);
         MC_FALLTHROUGH;
     case 6:
         widget_gotoyx (w, 6, 3);
-        tty_printf (_("Links:      %d"), (int) st.st_nlink);
+#ifdef ENABLE_EXT2FS_ATTR
+        if (!vfs_current_is_local ())
+            tty_print_string (_("Attributes: not supported"));
+        else
+        {
+            vfs_path_t *vpath;
+            unsigned long attr;
+
+            vpath = vfs_path_from_str (current_panel->dir.list[current_panel->selected].fname);
+
+            if (fgetflags (vfs_path_as_str (vpath), &attr) == 0)
+                tty_printf (_("Attributes: %s"), chattr_get_as_str (attr));
+            else
+                tty_print_string (_("Attributes: unavailable"));
+
+            vfs_path_free (vpath);
+        }
+#else
+        tty_print_string (_("Attributes: not supported"));
+#endif
         MC_FALLTHROUGH;
     case 5:
         widget_gotoyx (w, 5, 3);
