@@ -1,7 +1,7 @@
 /*
    Virtual File System: FTP file system.
 
-   Copyright (C) 1995-2019
+   Copyright (C) 1995-2020
    Free Software Foundation, Inc.
 
    Written by:
@@ -91,13 +91,14 @@ What to do with this?
 #endif
 #include <errno.h>
 #include <ctype.h>
-#include <sys/time.h>           /* gettimeofday() */
+#include <fcntl.h>
 #include <inttypes.h>           /* uintmax_t */
 
 #include "lib/global.h"
 #include "lib/util.h"
 #include "lib/strutil.h"        /* str_move() */
 #include "lib/mcconfig.h"
+#include "lib/timer.h"
 
 #include "lib/tty/tty.h"        /* enable/disable interrupt key */
 #include "lib/widget.h"         /* message() */
@@ -181,7 +182,7 @@ gboolean ftpfs_ignore_chattr_errors = TRUE;
    second parameter to ftpfs_changetype. */
 #define TYPE_UNKNOWN -1
 
-#define ABORT_TIMEOUT 5
+#define ABORT_TIMEOUT (5 * G_USEC_PER_SEC)
 /*** file scope type declarations ****************************************************************/
 
 #ifndef HAVE_SOCKLEN_T
@@ -1490,17 +1491,19 @@ ftpfs_linear_abort (struct vfs_class *me, vfs_file_handler_t * fh)
 
         if (select (dsock + 1, &mask, NULL, NULL, NULL) > 0)
         {
-            struct timeval start_tim;
+            guint64 start_tim;
             char buf[BUF_8K];
 
-            gettimeofday (&start_tim, NULL);
+            start_tim = mc_timer_elapsed (mc_global.timer);
+
             /* flush the remaining data */
             while (read (dsock, buf, sizeof (buf)) > 0)
             {
-                struct timeval tim;
+                guint64 tim;
 
-                gettimeofday (&tim, NULL);
-                if (tim.tv_sec > start_tim.tv_sec + ABORT_TIMEOUT)
+                tim = mc_timer_elapsed (mc_global.timer);
+
+                if (tim > start_tim + ABORT_TIMEOUT)
                 {
                     /* server keeps sending, drop the connection and ftpfs_reconnect */
                     close (dsock);
@@ -1745,8 +1748,7 @@ ftpfs_dir_load (struct vfs_class *me, struct vfs_s_inode *dir, char *remote_path
         return (-1);
     }
 
-    gettimeofday (&dir->timestamp, NULL);
-    dir->timestamp.tv_sec += ftpfs_directory_timeout;
+    dir->timestamp = mc_timer_elapsed (mc_global.timer) + ftpfs_directory_timeout * G_USEC_PER_SEC;
 
     if (ftp_super->strict == RFC_STRICT)
         sock = ftpfs_open_data_connection (me, super, "LIST", 0, TYPE_ASCII, 0);
