@@ -2,7 +2,7 @@
    Internal file viewer for the Midnight Commander
    Interface functions
 
-   Copyright (C) 1994-2019
+   Copyright (C) 1994-2020
    Free Software Foundation, Inc
 
    Written by:
@@ -201,9 +201,11 @@ mcview_new (int y, int x, int lines, int cols, gboolean is_panel)
     w = WIDGET (view);
     widget_init (w, y, x, lines, cols, mcview_callback, mcview_mouse_callback);
     w->options |= WOP_SELECTABLE | WOP_TOP_SELECT;
+    w->keymap = viewer_map;
 
     mcview_clear_mode_flags (&view->mode_flags);
     view->hexedit_mode = FALSE;
+    view->hex_keymap = viewer_hex_map;
     view->hexview_in_text = FALSE;
     view->locked = FALSE;
 
@@ -235,6 +237,7 @@ mcview_viewer (const char *command, const vfs_path_t * file_vpath, int start_lin
     WView *lc_mcview;
     WDialog *view_dlg;
     Widget *vw, *b;
+    WGroup *g;
 
     /* Create dialog and widgets, put them on the dialog */
     view_dlg = dlg_create (FALSE, 0, 0, 1, 1, WPOS_FULLSCREEN, FALSE, NULL, mcview_dialog_callback,
@@ -242,11 +245,13 @@ mcview_viewer (const char *command, const vfs_path_t * file_vpath, int start_lin
     vw = WIDGET (view_dlg);
     widget_want_tab (vw, TRUE);
 
+    g = GROUP (view_dlg);
+
     lc_mcview = mcview_new (vw->y, vw->x, vw->lines - 1, vw->cols, FALSE);
-    add_widget_autopos (view_dlg, lc_mcview, WPOS_KEEP_ALL, NULL);
+    group_add_widget_autopos (g, lc_mcview, WPOS_KEEP_ALL, NULL);
 
     b = WIDGET (buttonbar_new (TRUE));
-    add_widget_autopos (view_dlg, b, b->pos_flags, NULL);
+    group_add_widget_autopos (g, b, b->pos_flags, NULL);
 
     view_dlg->get_title = mcview_get_title;
 
