@@ -1,7 +1,7 @@
 /*
    lib - canonicalize path
 
-   Copyright (C) 2011-2019
+   Copyright (C) 2011-2020
    Free Software Foundation, Inc.
 
    Written by:
@@ -45,6 +45,7 @@ static struct vfs_class vfs_test_ops;
 static void
 setup (void)
 {
+    mc_global.timer = mc_timer_new ();
     str_init_strings (NULL);
 
     vfs_init ();
@@ -71,7 +72,9 @@ teardown (void)
 #endif
 
     vfs_shut ();
+
     str_uninit_strings ();
+    mc_timer_destroy (mc_global.timer);
 }
 
 /* --------------------------------------------------------------------------------------------- */
