@@ -1,7 +1,7 @@
 /*
    lib/vfs - test vfs_setup_cwd() functionality
 
-   Copyright (C) 2013-2019
+   Copyright (C) 2013-2020
    Free Software Foundation, Inc.
 
    Written by:
@@ -76,6 +76,7 @@ mc_stat (const vfs_path_t * vpath, struct stat *my_stat)
 static void
 setup (void)
 {
+    mc_global.timer = mc_timer_new ();
     str_init_strings (NULL);
 
     vfs_init ();
@@ -91,6 +92,7 @@ teardown (void)
 {
     vfs_shut ();
     str_uninit_strings ();
+    mc_timer_destroy (mc_global.timer);
 }
 
 /* --------------------------------------------------------------------------------------------- */
